/*!
 * Copyright Adaptavist 2022 (c) All rights reserved
 */
import { Headers, Request, Response, BaseApiCore, getUint8ArrayOfFormDataEntries } from '@managed-api/commons-core';
;
export class GenericAppApiCore extends BaseApiCore {
    constructor(options) {
        var _a, _b;
        super();
        this.options = options;
        if (!((_a = this.options) === null || _a === void 0 ? void 0 : _a.errorStrategy) && ((_b = this.options) === null || _b === void 0 ? void 0 : _b.errorStrategy) !== null) {
            this.setGlobalErrorStrategy((builder) => builder.retryOnRateLimiting(-1));
        }
    }
    setGlobalErrorStrategy(errorStrategy) {
        var _a;
        this.options = {
            ...(_a = this.options) !== null && _a !== void 0 ? _a : {},
            errorStrategy
        };
    }
    async fetch(resource, options) {
        var _a, _b;
        let request;
        if (resource instanceof Request) {
            request = resource;
        }
        else if ((options === null || options === void 0 ? void 0 : options.body) && typeof (options === null || options === void 0 ? void 0 : options.body['entries']) === 'function' && !ArrayBuffer.isView(options.body)) {
            const { boundary, uint8Array } = await getUint8ArrayOfFormDataEntries(this.implementation, options.body.entries());
            const formDataHeader = { 'Content-Type': `multipart/form-data; boundary=${boundary}` };
            request = new Request((_a = options === null || options === void 0 ? void 0 : options.method) !== null && _a !== void 0 ? _a : 'POST', resource, this.constructHeaders(options === null || options === void 0 ? void 0 : options.body, { ...formDataHeader, ...options === null || options === void 0 ? void 0 : options.headers }), uint8Array.buffer);
        }
        else {
            request = new Request((_b = options === null || options === void 0 ? void 0 : options.method) !== null && _b !== void 0 ? _b : 'GET', resource, this.constructHeaders(options === null || options === void 0 ? void 0 : options.body, options === null || options === void 0 ? void 0 : options.headers), options === null || options === void 0 ? void 0 : options.body);
        }
        request.setBufferImplementation(this.implementation);
        return await this.implementation.performHttpCall(request);
    }
    constructHeaders(body, manualHeaders) {
        const headers = manualHeaders ? new Headers(manualHeaders) : new Headers();
        if (body && !headers.get('Content-Type')) {
            if (typeof body['arrayBuffer'] === 'function' && !!body.type) {
                const type = body.type;
                headers.append('Content-Type', type);
            }
            else {
                headers.append('Content-Type', 'application/json; charset=utf-8');
            }
        }
        return headers;
    }
    buildResponse(url, status, statusText, headers, body) {
        const response = new Response(url, status, statusText, headers, body);
        response.setBufferImplementation(this.implementation);
        return response;
    }
    getGlobalErrorStrategy() {
        var _a;
        return (_a = this.options) === null || _a === void 0 ? void 0 : _a.errorStrategy;
    }
}
